@@ -22,7 +22,7 @@
 #ifndef CAPNP_COMPILER_MODULE_LOADER_H_
 #define CAPNP_COMPILER_MODULE_LOADER_H_
 
-#if defined(__GNUC__) && !CAPNP_HEADER_WARNINGS
+#if defined(__GNUC__) && !defined(CAPNP_HEADER_WARNINGS)
 #pragma GCC system_header
 #endif
 
