@@ -22,7 +22,7 @@
 #ifndef CAPNP_PRETTY_PRINT_H_
 #define CAPNP_PRETTY_PRINT_H_
 
-#if defined(__GNUC__) && !CAPNP_HEADER_WARNINGS
+#if defined(__GNUC__) && !defined(CAPNP_HEADER_WARNINGS)
 #pragma GCC system_header
 #endif
 
